@@ -2,7 +2,7 @@ from ssl import SSLContext
 from typing import Tuple
 
 from .._exceptions import ProxyError
-from .._types import URL, Headers, Origin, TimeoutDict
+from .._types import URL, Headers, TimeoutDict
 from .._utils import get_logger, url_to_origin
 from .base import AsyncByteStream
 from .connection import AsyncHTTPConnection
@@ -15,7 +15,8 @@ def merge_headers(
     default_headers: Headers = None, override_headers: Headers = None
 ) -> Headers:
     """
-    Append default_headers and override_headers, de-duplicating if a key existing in both cases.
+    Append default_headers and override_headers, de-duplicating if a key existing in
+    both cases.
     """
     default_headers = [] if default_headers is None else default_headers
     override_headers = [] if override_headers is None else override_headers
@@ -44,8 +45,8 @@ class AsyncHTTPProxy(AsyncConnectionPool):
     verifying connections.
     * **max_connections** - `Optional[int]` - The maximum number of concurrent
     connections to allow.
-    * **max_keepalive** - `Optional[int]` - The maximum number of connections
-    to allow before closing keep-alive connections.
+    * **max_keepalive_connections** - `Optional[int]` - The maximum number of
+    connections to allow before closing keep-alive connections.
     * **http2** - `bool` - Enable HTTP/2 support.
     """
 
@@ -56,9 +57,11 @@ class AsyncHTTPProxy(AsyncConnectionPool):
         proxy_mode: str = "DEFAULT",
         ssl_context: SSLContext = None,
         max_connections: int = None,
-        max_keepalive: int = None,
+        max_keepalive_connections: int = None,
         keepalive_expiry: float = None,
         http2: bool = False,
+        # Deprecated argument style:
+        max_keepalive: int = None,
     ):
         assert proxy_mode in ("DEFAULT", "FORWARD_ONLY", "TUNNEL_ONLY")
 
@@ -68,9 +71,10 @@ class AsyncHTTPProxy(AsyncConnectionPool):
         super().__init__(
             ssl_context=ssl_context,
             max_connections=max_connections,
-            max_keepalive=max_keepalive,
+            max_keepalive_connections=max_keepalive_connections,
             keepalive_expiry=keepalive_expiry,
             http2=http2,
+            max_keepalive=max_keepalive,
         )
 
     async def request(
@@ -146,13 +150,21 @@ class AsyncHTTPProxy(AsyncConnectionPool):
         url = self.proxy_origin + (target,)
         headers = merge_headers(self.proxy_headers, headers)
 
-        response = await connection.request(
+        (
+            http_version,
+            status_code,
+            reason_phrase,
+            headers,
+            stream,
+        ) = await connection.request(
             method, url, headers=headers, stream=stream, timeout=timeout
         )
+
         wrapped_stream = ResponseByteStream(
-            response[4], connection=connection, callback=self._response_closed
+            stream, connection=connection, callback=self._response_closed
         )
-        return response[0], response[1], response[2], response[3], wrapped_stream
+
+        return http_version, status_code, reason_phrase, headers, wrapped_stream
 
     async def _tunnel_request(
         self,
@@ -170,6 +182,8 @@ class AsyncHTTPProxy(AsyncConnectionPool):
         connection = await self._get_connection_from_pool(origin)
 
         if connection is None:
+            scheme, host, port = origin
+
             # First, create a connection to the proxy server
             proxy_connection = AsyncHTTPConnection(
                 origin=self.proxy_origin,
@@ -181,19 +195,19 @@ class AsyncHTTPProxy(AsyncConnectionPool):
 
             # CONNECT www.example.org:80 HTTP/1.1
             # [proxy-headers]
-            if url[2] is None:
-                target = url[1]
-            else:
-                target = b"%b:%d" % (url[1], url[2])
+            target = b"%b:%d" % (host, port)
             connect_url = self.proxy_origin + (target,)
             connect_headers = [(b"Host", target), (b"Accept", b"*/*")]
             connect_headers = merge_headers(connect_headers, self.proxy_headers)
-            proxy_response = await proxy_connection.request(
+            (
+                _,
+                proxy_status_code,
+                proxy_reason_phrase,
+                _,
+                proxy_stream,
+            ) = await proxy_connection.request(
                 b"CONNECT", connect_url, headers=connect_headers, timeout=timeout
             )
-            proxy_status_code = proxy_response[1]
-            proxy_reason_phrase = proxy_response[2]
-            proxy_stream = proxy_response[4]
             logger.trace(
                 "tunnel_response proxy_status_code=%r proxy_reason=%r ",
                 proxy_status_code,
@@ -210,8 +224,8 @@ class AsyncHTTPProxy(AsyncConnectionPool):
 
             # Upgrade to TLS if required
             # We assume the target speaks TLS on the specified port
-            if url[0] == b"https":
-                await proxy_connection.start_tls(url[1], timeout)
+            if scheme == b"https":
+                await proxy_connection.start_tls(host, timeout)
 
             # The CONNECT request is successful, so we have now SWITCHED PROTOCOLS.
             # This means the proxy connection is now unusable, and we must create
@@ -227,10 +241,18 @@ class AsyncHTTPProxy(AsyncConnectionPool):
 
         # Once the connection has been established we can send requests on
         # it as normal.
-        response = await connection.request(
+        (
+            http_version,
+            status_code,
+            reason_phrase,
+            headers,
+            stream,
+        ) = await connection.request(
             method, url, headers=headers, stream=stream, timeout=timeout,
         )
+
         wrapped_stream = ResponseByteStream(
-            response[4], connection=connection, callback=self._response_closed
+            stream, connection=connection, callback=self._response_closed
         )
-        return response[0], response[1], response[2], response[3], wrapped_stream
+
+        return http_version, status_code, reason_phrase, headers, wrapped_stream
