@@ -4,10 +4,12 @@ from typing import Iterator, List, Tuple, Union
 import h11
 
 from .._backends.auto import SyncSocketStream
-from .._exceptions import ProtocolError, map_exceptions
+from .._bytestreams import IteratorByteStream, PlainByteStream
+from .._exceptions import LocalProtocolError, RemoteProtocolError, map_exceptions
 from .._types import URL, Headers, TimeoutDict
 from .._utils import get_logger
-from .base import SyncByteStream, SyncHTTPTransport, ConnectionState
+from .base import SyncByteStream, ConnectionState
+from .http import SyncBaseHTTPConnection
 
 H11Event = Union[
     h11.Request,
@@ -21,8 +23,8 @@ H11Event = Union[
 logger = get_logger(__name__)
 
 
-class SyncHTTP11Connection(SyncHTTPTransport):
-    READ_NUM_BYTES = 4096
+class SyncHTTP11Connection(SyncBaseHTTPConnection):
+    READ_NUM_BYTES = 64 * 1024
 
     def __init__(
         self, socket: SyncSocketStream, ssl_context: SSLContext = None,
@@ -34,6 +36,15 @@ class SyncHTTP11Connection(SyncHTTPTransport):
 
         self.state = ConnectionState.ACTIVE
 
+    def __repr__(self) -> str:
+        return f"<SyncHTTP11Connection state={self.state}>"
+
+    def info(self) -> str:
+        return f"HTTP/1.1, {self.state.name}"
+
+    def get_state(self) -> ConnectionState:
+        return self.state
+
     def mark_as_ready(self) -> None:
         if self.state == ConnectionState.IDLE:
             self.state = ConnectionState.READY
@@ -47,7 +58,7 @@ class SyncHTTP11Connection(SyncHTTPTransport):
         timeout: TimeoutDict = None,
     ) -> Tuple[bytes, int, bytes, List[Tuple[bytes, bytes]], SyncByteStream]:
         headers = [] if headers is None else headers
-        stream = SyncByteStream() if stream is None else stream
+        stream = PlainByteStream(b"") if stream is None else stream
         timeout = {} if timeout is None else timeout
 
         self.state = ConnectionState.ACTIVE
@@ -60,15 +71,18 @@ class SyncHTTP11Connection(SyncHTTPTransport):
             reason_phrase,
             headers,
         ) = self._receive_response(timeout)
-        stream = SyncByteStream(
+        response_stream = IteratorByteStream(
             iterator=self._receive_response_data(timeout),
             close_func=self._response_closed,
         )
-        return (http_version, status_code, reason_phrase, headers, stream)
+        return (http_version, status_code, reason_phrase, headers, response_stream)
 
-    def start_tls(self, hostname: bytes, timeout: TimeoutDict = None) -> None:
+    def start_tls(
+        self, hostname: bytes, timeout: TimeoutDict = None
+    ) -> SyncSocketStream:
         timeout = {} if timeout is None else timeout
         self.socket = self.socket.start_tls(hostname, self.ssl_context, timeout)
+        return self.socket
 
     def _send_request(
         self, method: bytes, url: URL, headers: Headers, timeout: TimeoutDict,
@@ -78,7 +92,8 @@ class SyncHTTP11Connection(SyncHTTPTransport):
         """
         logger.trace("send_request method=%r url=%r headers=%s", method, url, headers)
         _scheme, _host, _port, target = url
-        event = h11.Request(method=method, target=target, headers=headers)
+        with map_exceptions({h11.LocalProtocolError: LocalProtocolError}):
+            event = h11.Request(method=method, target=target, headers=headers)
         self._send_event(event, timeout)
 
     def _send_request_body(
@@ -138,7 +153,7 @@ class SyncHTTP11Connection(SyncHTTPTransport):
         Read a single `h11` event, reading more data from the network if needed.
         """
         while True:
-            with map_exceptions({h11.RemoteProtocolError: ProtocolError}):
+            with map_exceptions({h11.RemoteProtocolError: RemoteProtocolError}):
                 event = self.h11_state.next_event()
 
             if event is h11.NEED_DATA:
